# Copyright 2019 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import ipaddress

from oslo_log import log as logging
from tempest.common import waiters
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

from neutron_tempest_plugin import config
from neutron_tempest_plugin.scenario import base
from neutron_tempest_plugin.scenario import constants as const

LOG = logging.getLogger(__name__)
CONF = config.CONF


class PortsTest(base.BaseTempestTestCase):
    credentials = ['primary', 'admin']

    @classmethod
    def resource_setup(cls):
        super(PortsTest, cls).resource_setup()
        # setup basic topology for servers we can log into it
        cls.router = cls.create_router_by_client()
        cls.keypair = cls.create_keypair()
        cls.secgroup = cls.create_security_group(
            name=data_utils.rand_name("test_port_secgroup"))
        cls.create_loginable_secgroup_rule(
            secgroup_id=cls.secgroup['id'])
        cls.create_pingable_secgroup_rule(
            secgroup_id=cls.secgroup['id'])
        cls.network = cls.create_network()
        cls.subnet = cls.create_subnet(cls.network)
        cls.create_router_interface(cls.router['id'], cls.subnet['id'])
        cls.port = cls.create_port(cls.network,
                                   name=data_utils.rand_name("port"),
                                   security_groups=[cls.secgroup['id']])

    def _create_instance_with_port(self, port):
        """Create instance for port testing

        :param port (object): the port used
        """
        servers, fips = ([], [])
        server_args = {
            'flavor_ref': CONF.compute.flavor_ref,
            'image_ref': CONF.compute.image_ref,
            'key_name': self.keypair['name'],
            'networks': [{'port': port['id']}]
        }
        servers.append(self.create_server(**server_args))
        waiters.wait_for_server_status(
            self.os_primary.servers_client, servers[0]['server']['id'],
            const.SERVER_STATUS_ACTIVE)
        fips.append(self.create_floatingip(port=port))
        return fips, servers

    @decorators.idempotent_id('5500797e-b8c2-4e07-a5e0-89fa4e814965')
    def test_previously_used_port(self):
        for i in range(2):
            fips, servers = self._create_instance_with_port(
                self.port)
            self.check_connectivity(fips[0]['floating_ip_address'],
                                    CONF.validation.image_ssh_user,
                                    self.keypair['private_key'])
            self.os_primary.servers_client.delete_server(
                servers[0]['server']['id'])
            waiters.wait_for_server_termination(
                self.os_primary.servers_client,
                servers[0]['server']['id'])
            self._try_delete_resource(self.delete_floatingip, fips[0])

    @decorators.idempotent_id('62e32802-1d21-11eb-b322-74e5f9e2a801')
    def test_port_with_fixed_ip(self):
        """Test scenario:

        1) Get the last IP from the range of Subnet "Allocation pool"
        2) Create Port with fixed_ip resolved in #1
        3) Create a VM using updated Port in #2 and add Floating IP
        4) Check SSH access to VM
        """
        ip_range = [str(ip) for ip in ipaddress.IPv4Network(
            self.subnet['cidr'])]
        # Because of the tests executed in Parallel the IP may already
        # be in use, so we'll try using IPs from Allocation pool
        # (in reverse order) up until Port is successfully created.
        for ip in reversed(ip_range):
            try:
                port = self.create_port(
                    self.network,
                    name=data_utils.rand_name("fixed_ip_port"),
                    security_groups=[self.secgroup['id']],
                    fixed_ips=[{'ip_address': ip}])
                if port is not None:
                    break
            except Exception as e:
                LOG.warning('Failed to create Port, using Fixed_IP:{}, '
                            'the Error was:{}'.format(ip, e))
        fip, server = self._create_instance_with_port(port)
        self.check_connectivity(fip[0]['floating_ip_address'],
                                CONF.validation.image_ssh_user,
                                self.keypair['private_key'])



# Copyright 2016 Hewlett Packard Enterprise Development Company LP
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import netaddr

from tempest.common import utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

from neutron_tempest_plugin.neutron_dynamic_routing.api import\
    test_bgp_speaker_extensions as test_base


class BgpSpeakerTestJSONNegative(test_base.BgpSpeakerTestJSONBase):

    """Negative test cases asserting proper behavior of BGP API extension"""

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('75e9ee2f-6efd-4320-bff7-ae24741c8b06')
    def test_create_bgp_speaker_illegal_local_asn(self):
        wrong_asn = 65537
        if utils.is_extension_enabled('bgp_4byte_asn', 'network'):
            wrong_asn = 4294967296
        self.assertRaises(lib_exc.BadRequest,
                          self.create_bgp_speaker,
                          local_as=wrong_asn)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('6742ec2e-382a-4453-8791-13a19b47cd13')
    def test_create_bgp_speaker_non_admin(self):
        self.assertRaises(lib_exc.Forbidden,
                          self.bgp_client.create_bgp_speaker,
                          {'bgp_speaker': self.default_bgp_speaker_args})

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('33f7aaf0-9786-478b-b2d1-a51086a50eb4')
    def test_create_bgp_peer_non_admin(self):
        self.assertRaises(lib_exc.Forbidden,
                          self.bgp_client.create_bgp_peer,
                          {'bgp_peer': self.default_bgp_peer_args})

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('39435932-0266-4358-899b-0e9b1e53c3e9')
    def test_update_bgp_speaker_local_asn(self):
        bgp_speaker = self.create_bgp_speaker(**self.default_bgp_speaker_args)
        bgp_speaker_id = bgp_speaker['id']

        self.assertRaises(lib_exc.BadRequest, self.update_bgp_speaker,
                          bgp_speaker_id, local_as='4321')

    @decorators.idempotent_id('9cc33701-51e5-421f-a5d5-fd7b330e550f')
    def test_get_advertised_routes_tenant_networks(self):
        addr_scope1 = self.create_address_scope('my-scope1', ip_version=4)
        addr_scope2 = self.create_address_scope('my-scope2', ip_version=4)
        ext_net = self.create_shared_network(**{'router:external': True})
        tenant_net1 = self.create_network()
        tenant_net2 = self.create_network()
        ext_subnetpool = self.create_subnetpool(
                                           'test-pool-ext',
                                           is_admin=True,
                                           default_prefixlen=24,
                                           address_scope_id=addr_scope1['id'],
                                           prefixes=['8.0.0.0/8'])
        tenant_subnetpool1 = self.create_subnetpool(
                                           'tenant-test-pool',
                                           default_prefixlen=25,
                                           address_scope_id=addr_scope1['id'],
                                           prefixes=['10.10.0.0/16'])
        tenant_subnetpool2 = self.create_subnetpool(
                                           'tenant-test-pool',
                                           default_prefixlen=25,
                                           address_scope_id=addr_scope2['id'],
                                           prefixes=['11.10.0.0/16'])
        self.create_subnet({'id': ext_net['id']},
                           cidr=netaddr.IPNetwork('8.0.0.0/24'),
                           ip_version=4,
                           client=self.admin_client,
                           subnetpool_id=ext_subnetpool['id'])
        tenant_subnet1 = self.create_subnet(
                                       {'id': tenant_net1['id']},
                                       cidr=netaddr.IPNetwork('10.10.0.0/24'),
                                       ip_version=4,
                                       subnetpool_id=tenant_subnetpool1['id'])
        tenant_subnet2 = self.create_subnet(
                                       {'id': tenant_net2['id']},
                                       cidr=netaddr.IPNetwork('11.10.0.0/24'),
                                       ip_version=4,
                                       subnetpool_id=tenant_subnetpool2['id'])
        ext_gw_info = {'network_id': ext_net['id']}
        router = self.create_router(ext_gw_info)
        self.admin_routers.append(router)
        self.admin_client.add_router_interface_with_subnet_id(
                                                       router['id'],
                                                       tenant_subnet1['id'])
        self.admin_routerports.append({'router_id': router['id'],
                                       'subnet_id': tenant_subnet1['id']})
        self.admin_client.add_router_interface_with_subnet_id(
                                                       router['id'],
                                                       tenant_subnet2['id'])
        self.admin_routerports.append({'router_id': router['id'],
                                       'subnet_id': tenant_subnet2['id']})
        bgp_speaker = self.create_bgp_speaker(**self.default_bgp_speaker_args)
        bgp_speaker_id = bgp_speaker['id']
        self.bgp_adm_client.add_bgp_gateway_network(bgp_speaker_id,
                                                    ext_net['id'])
        routes = self.bgp_adm_client.get_bgp_advertised_routes(bgp_speaker_id)
        self.assertEqual(1, len(routes['advertised_routes']))
        self.assertEqual(tenant_subnet1['cidr'],
                         routes['advertised_routes'][0]['destination'])
        fixed_ip = router['external_gateway_info']['external_fixed_ips'][0]
        self.assertEqual(fixed_ip['ip_address'],
                         routes['advertised_routes'][0]['next_hop'])



# Copyright 2014 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

from neutron_lib import constants as lib_constants
from tempest.common import utils
from tempest.lib import decorators

from neutron_tempest_plugin.api import base


class PortsTestJSON(base.BaseNetworkTest):

    @classmethod
    def resource_setup(cls):
        super(PortsTestJSON, cls).resource_setup()
        cls.network = cls.create_network()

    def _confirm_dns_assignment(self, port):
        # NOTE(manjeets) port created with single subnet
        # would have only one dns_assignment
        dns_assignment = port['dns_assignment'][0]
        ip = port['fixed_ips'][0]['ip_address']
        if port['dns_name']:
            hostname = port['dns_name']
        else:
            hostname = 'host-%s' % ip.replace('.', '-')
        self.assertEqual(hostname, dns_assignment['hostname'])

        # To avoid hard coding the expected dns_domain value
        # in neutron.conf we just check that the fqdn starts
        # with correct hostname
        self.assertTrue(dns_assignment['fqdn'].startswith(hostname))
        self.assertEqual(ip, dns_assignment['ip_address'])

    @decorators.idempotent_id('c72c1c0c-2193-4aca-bbb4-b1442640bbbb')
    @utils.requires_ext(extension="standard-attr-description",
                       service="network")
    def test_create_update_port_description(self):
        body = self.create_port(self.network,
                                description='d1')
        self.assertEqual('d1', body['description'])
        body = self.client.list_ports(id=body['id'])['ports'][0]
        self.assertEqual('d1', body['description'])
        body = self.client.update_port(body['id'],
                                       description='d2')
        self.assertEqual('d2', body['port']['description'])
        body = self.client.list_ports(id=body['port']['id'])['ports'][0]
        self.assertEqual('d2', body['description'])

    @decorators.idempotent_id('3ae162e8-ff00-490c-a423-6a88e48f1ed6')
    def test_create_update_port_security(self):
        body = self.create_port(self.network,
                                port_security_enabled=True)
        self.assertTrue(body['port_security_enabled'])
        body = self.client.list_ports(id=body['id'])['ports'][0]
        self.assertTrue(body['port_security_enabled'])
        body = self.client.update_port(body['id'],
                                       port_security_enabled=False,
                                       security_groups=[])
        self.assertFalse(body['port']['port_security_enabled'])
        body = self.client.list_ports(id=body['port']['id'])['ports'][0]
        self.assertFalse(body['port_security_enabled'])

    @decorators.idempotent_id('539fbefe-fb36-48aa-9a53-8c5fbd44e492')
    @utils.requires_ext(extension="dns-integration",
                       service="network")
    def test_create_update_port_with_dns_name(self):
        # NOTE(manjeets) dns_domain is set to openstackgate.local
        # (or any other configured value) so dns_name for port can be set
        self.create_subnet(self.network)
        body = self.create_port(self.network, dns_name='d1')
        self.assertEqual('d1', body['dns_name'])
        self._confirm_dns_assignment(body)
        body = self.client.list_ports(id=body['id'])['ports'][0]
        self._confirm_dns_assignment(body)
        self.assertEqual('d1', body['dns_name'])
        body = self.client.update_port(body['id'],
                                       dns_name='d2')
        self.assertEqual('d2', body['port']['dns_name'])
        self._confirm_dns_assignment(body['port'])
        body = self.client.show_port(body['port']['id'])['port']
        self.assertEqual('d2', body['dns_name'])
        self._confirm_dns_assignment(body)

    @decorators.idempotent_id('435e89df-a8bb-4b41-801a-9f20d362d777')
    @utils.requires_ext(extension="dns-integration",
                       service="network")
    def test_create_update_port_with_no_dns_name(self):
        self.create_subnet(self.network)
        body = self.create_port(self.network)
        self.assertFalse(body['dns_name'])
        self._confirm_dns_assignment(body)
        port_body = self.client.show_port(body['id'])
        self.assertFalse(port_body['port']['dns_name'])
        self._confirm_dns_assignment(port_body['port'])

    @decorators.idempotent_id('dfe8cc79-18d9-4ae8-acef-3ec6bb719aa7')
    @utils.requires_ext(extension="dns-domain-ports",
                       service="network")
    def test_create_update_port_with_dns_domain(self):
        self.create_subnet(self.network)
        body = self.create_port(self.network, dns_name='d1',
                                dns_domain='test.org.')
        self.assertEqual('d1', body['dns_name'])
        self.assertEqual('test.org.', body['dns_domain'])
        self._confirm_dns_assignment(body)
        body = self.client.list_ports(id=body['id'])['ports'][0]
        self._confirm_dns_assignment(body)
        self.assertEqual('d1', body['dns_name'])
        self.assertEqual('test.org.', body['dns_domain'])
        body = self.client.update_port(body['id'],
                                       dns_name='d2', dns_domain='d.org.')
        self.assertEqual('d2', body['port']['dns_name'])
        self.assertEqual('d.org.', body['port']['dns_domain'])
        self._confirm_dns_assignment(body['port'])
        body = self.client.show_port(body['port']['id'])['port']
        self.assertEqual('d2', body['dns_name'])
        self.assertEqual('d.org.', body['dns_domain'])
        self._confirm_dns_assignment(body)

    @decorators.idempotent_id('c72c1c0c-2193-4aca-bbb4-b1442640c123')
    def test_change_dhcp_flag_then_create_port(self):
        s = self.create_subnet(self.network, enable_dhcp=False)
        self.create_port(self.network)
        self.client.update_subnet(s['id'], enable_dhcp=True)
        self.create_port(self.network)

    @decorators.idempotent_id('1d6d8683-8691-43c6-a7ba-c69723258726')
    def test_add_ips_to_port(self):
        s = self.create_subnet(self.network)
        port = self.create_port(self.network)
        # request another IP on the same subnet
        port['fixed_ips'].append({'subnet_id': s['id']})
        updated = self.client.update_port(port['id'],
                                          fixed_ips=port['fixed_ips'])
        subnets = [ip['subnet_id'] for ip in updated['port']['fixed_ips']]
        expected = [s['id'], s['id']]
        self.assertEqual(expected, subnets)


class PortsIpv6TestJSON(base.BaseNetworkTest):

    _ip_version = lib_constants.IP_VERSION_6

    @classmethod
    def resource_setup(cls):
        super(PortsIpv6TestJSON, cls).resource_setup()
        cls.network = cls.create_network()

    @decorators.idempotent_id('b85879fb-4852-4b99-aa32-3f8a7a6a3f01')
    def test_add_ipv6_ips_to_port(self):
        s = self.create_subnet(self.network, ip_version=self._ip_version)
        port = self.create_port(self.network)
        # request another IP on the same subnet
        port['fixed_ips'].append({'subnet_id': s['id']})
        updated = self.client.update_port(port['id'],
                                          fixed_ips=port['fixed_ips'])
        subnets = [ip['subnet_id'] for ip in updated['port']['fixed_ips']]
        expected = [s['id'], s['id']]
        self.assertEqual(expected, subnets)


class PortsSearchCriteriaTest(base.BaseSearchCriteriaTest):

    resource = 'port'

    @classmethod
    def resource_setup(cls):
        super(PortsSearchCriteriaTest, cls).resource_setup()
        net = cls.create_network(network_name='port-search-test-net')
        for name in cls.resource_names:
            cls.create_port(net, name=name)

    @decorators.idempotent_id('9ab73df4-960a-4ae3-87d3-60992b8d3e2d')
    def test_list_sorts_asc(self):
        self._test_list_sorts_asc()

    @decorators.idempotent_id('b426671d-7270-430f-82ff-8f33eec93010')
    def test_list_sorts_desc(self):
        self._test_list_sorts_desc()

    @decorators.idempotent_id('a202fdc8-6616-45df-b6a0-463932de6f94')
    def test_list_pagination(self):
        self._test_list_pagination()

    @decorators.idempotent_id('f4723b8e-8186-4b9a-bf9e-57519967e048')
    def test_list_pagination_with_marker(self):
        self._test_list_pagination_with_marker()

    @decorators.idempotent_id('fcd02a7a-f07e-4d5e-b0ca-b58e48927a9b')
    def test_list_pagination_with_href_links(self):
        self._test_list_pagination_with_href_links()

    @decorators.idempotent_id('3afe7024-77ab-4cfe-824b-0b2bf4217727')
    def test_list_no_pagination_limit_0(self):
        self._test_list_no_pagination_limit_0()

    @decorators.idempotent_id('b8857391-dc44-40cc-89b7-2800402e03ce')
    def test_list_pagination_page_reverse_asc(self):
        self._test_list_pagination_page_reverse_asc()

    @decorators.idempotent_id('4e51e9c9-ceae-4ec0-afd4-147569247699')
    def test_list_pagination_page_reverse_desc(self):
        self._test_list_pagination_page_reverse_desc()

    @decorators.idempotent_id('74293e59-d794-4a93-be09-38667199ef68')
    def test_list_pagination_page_reverse_with_href_links(self):
        self._test_list_pagination_page_reverse_with_href_links()


class PortsTaggingOnCreationTestJSON(base.BaseNetworkTest):

    _tags = [
        ['tag-1', 'tag-2', 'tag-3'],
        ['tag-1', 'tag-2'],
        ['tag-1', 'tag-3'],
        []
    ]

    @classmethod
    def resource_setup(cls):
        super(PortsTaggingOnCreationTestJSON, cls).resource_setup()
        cls.network = cls.create_network()

    def _create_ports_in_bulk(self, ports):
        body = self.client.create_bulk_port(ports)
        for port in body['ports']:
            self.ports.append(port)
        return body

    def _create_ports_list(self):
        num_ports = len(self._tags)
        net_id = self.network['id']
        port = {'port': {'network_id': net_id,
                         'admin_state_up': True}}
        return [copy.deepcopy(port) for x in range(num_ports)]

    @decorators.idempotent_id('5cf26014-fdd3-4a6d-b94d-a05f0c55da89')
    @utils.requires_ext(extension="tag-ports-during-bulk-creation",
                        service="network")
    def test_tagging_ports_during_bulk_creation(self):
        ports = self._create_ports_list()
        ports_tags_map = {}
        for port, tags in zip(ports, self._tags):
            port['port']['tags'] = tags
            port['port']['name'] = '-'.join(tags)
            ports_tags_map[port['port']['name']] = tags
        body = self._create_ports_in_bulk(ports)
        for port in body['ports']:
            self.assertEqual(ports_tags_map[port['name']], port['tags'])

    @decorators.idempotent_id('33eda785-a08a-44a0-1bbb-fb50a2f1cd78')
    @utils.requires_ext(extension="tag-ports-during-bulk-creation",
                        service="network")
    def test_tagging_ports_during_bulk_creation_no_tags(self):
        ports = self._create_ports_list()
        body = self._create_ports_in_bulk(ports)
        for port in body['ports']:
            self.assertFalse(port['tags'])

    @decorators.idempotent_id('6baa43bf-88fb-8bca-6051-97ea1a5e8f4f')
    @utils.requires_ext(extension="tag-ports-during-bulk-creation",
                        service="network")
    def test_tagging_ports_during_creation(self):
        port = {'name': 'port', 'tags': self._tags[0]}
        body = self.create_port(self.network, **port)
        self.assertEqual(self._tags[0], body['tags'])



# Copyright 2013 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import netaddr
from tempest.common import utils
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

from neutron_tempest_plugin.api import base
from neutron_tempest_plugin import config

CONF = config.CONF


class FloatingIPTestJSON(base.BaseNetworkTest):

    required_extensions = ['router']

    @classmethod
    def resource_setup(cls):
        super(FloatingIPTestJSON, cls).resource_setup()
        cls.ext_net_id = CONF.network.public_network_id

        # Create network, subnet, router and add interface
        cls.network = cls.create_network()
        cls.subnet = cls.create_subnet(cls.network)
        cls.router = cls.create_router(data_utils.rand_name('router'),
                                       external_network_id=cls.ext_net_id)
        cls.create_router_interface(cls.router['id'], cls.subnet['id'])
        cls.port = list()
        # Create two ports one each for Creation and Updating of floatingIP
        for i in range(2):
            cls.create_port(cls.network)

    @decorators.idempotent_id('f6a0fb6c-cb64-4b81-b0d5-f41d8f69d22d')
    def test_blank_update_clears_association(self):
        # originally the floating IP had no attributes other than its
        # association, so an update with an empty body was a signal to
        # clear the association. This test ensures we maintain that behavior.
        body = self.create_floatingip(port=self.ports[0])
        self.assertEqual(self.ports[0]['id'], body['port_id'])
        body = self.client.update_floatingip(body['id'])['floatingip']
        self.assertFalse(body['port_id'])

    @decorators.idempotent_id('c72c1c0c-2193-4aca-eeee-b1442641ffff')
    @utils.requires_ext(extension="standard-attr-description",
                       service="network")
    def test_create_update_floatingip_description(self):
        body = self.create_floatingip(port=self.ports[0], description='d1')
        self.assertEqual('d1', body['description'])
        body = self.client.show_floatingip(body['id'])['floatingip']
        self.assertEqual('d1', body['description'])
        body = self.client.update_floatingip(body['id'], description='d2')
        self.assertEqual('d2', body['floatingip']['description'])
        body = self.client.show_floatingip(body['floatingip']['id'])
        self.assertEqual('d2', body['floatingip']['description'])
        # disassociate
        body = self.client.update_floatingip(body['floatingip']['id'],
                                             port_id=None)
        self.assertEqual('d2', body['floatingip']['description'])

    @decorators.idempotent_id('fd7161e1-2167-4686-a6ff-0f3df08001bb')
    @utils.requires_ext(extension="standard-attr-description",
                       service="network")
    def test_floatingip_update_extra_attributes_port_id_not_changed(self):
        port_id = self.ports[1]['id']
        body = self.create_floatingip(port_id=port_id, description='d1')
        self.assertEqual('d1', body['description'])
        body = self.client.show_floatingip(body['id'])['floatingip']
        self.assertEqual(port_id, body['port_id'])
        # Update description
        body = self.client.update_floatingip(body['id'], description='d2')
        self.assertEqual('d2', body['floatingip']['description'])
        # Floating IP association is not changed.
        self.assertEqual(port_id, body['floatingip']['port_id'])
        body = self.client.show_floatingip(body['floatingip']['id'])
        self.assertEqual('d2', body['floatingip']['description'])
        self.assertEqual(port_id, body['floatingip']['port_id'])
        # disassociate
        body = self.client.update_floatingip(body['floatingip']['id'],
                                             port_id=None)
        self.assertIsNone(body['floatingip']['port_id'])

    @decorators.idempotent_id('cecae820-ebaa-4f96-b386-6a9fbf25c552')
    @utils.requires_ext(extension="standard-attr-description",
                        service="network")
    @utils.requires_ext(extension="fip-port-details", service="network")
    def test_create_update_floatingip_port_details(self):

        body = self.create_floatingip(port=self.ports[0], description='d1')
        self._assert_port_details(self.ports[0], body)
        body = self.client.show_floatingip(body['id'])['floatingip']
        self._assert_port_details(self.ports[0], body)
        body = self.client.update_floatingip(body['id'], description='d2')
        self._assert_port_details(self.ports[0], body['floatingip'])
        # disassociate
        body = self.client.update_floatingip(body['floatingip']['id'],
                                             port_id=None)
        self.assertIn('port_details', body['floatingip'])
        self.assertIsNone(body['floatingip']['port_details'])

    def _assert_port_details(self, port, body):
        self.assertIn('port_details', body)
        port_details = body['port_details']
        self.assertEqual(port['name'], port_details['name'])
        self.assertEqual(port['network_id'], port_details['network_id'])
        self.assertEqual(port['mac_address'], port_details['mac_address'])
        self.assertEqual(port['admin_state_up'],
                         port_details['admin_state_up'])
        self.assertEqual(port['status'], port_details['status'])
        self.assertEqual(port['device_id'], port_details['device_id'])
        self.assertEqual(port['device_owner'], port_details['device_owner'])


class FloatingIPPoolTestJSON(base.BaseAdminNetworkTest):

    required_extensions = ['router']

    @decorators.idempotent_id('6c438332-4554-461c-9668-512ae09bf952')
    @utils.requires_ext(extension="floatingip-pools", service="network")
    def test_create_floatingip_from_specific_pool(self):
        network = self.create_network(client=self.admin_client, external=True)
        subnet1 = self.create_subnet(network, client=self.admin_client)
        subnet2 = self.create_subnet(network, client=self.admin_client)
        pools = self.client.list_floatingip_pools()["floatingip_pools"]

        def test_create_floatingip_from_subnet(pools, subnet):
            pool = None
            for p in pools:
                if p['network_id'] == subnet['network_id'] \
                        and p['subnet_id'] == subnet['id']:
                    pool = p
                    break

            self.assertTrue(pool)
            new_floatingip = self.create_floatingip(
                pool['network_id'], subnet_id=pool['subnet_id'])
            cidr = netaddr.IPNetwork(pool['cidr'])
            ip_address = netaddr.IPAddress(
                new_floatingip['floating_ip_address'])
            self.assertIn(ip_address, cidr)
            fip_id = new_floatingip['id']
            floatingip = self.client.get_floatingip(fip_id)['floatingip']
            self.assertEqual(new_floatingip['floating_ip_address'],
                             floatingip['floating_ip_address'])

        test_create_floatingip_from_subnet(pools, subnet1)
        test_create_floatingip_from_subnet(pools, subnet2)



# Copyright 2018 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_log import log
from tempest.common import utils
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

from neutron_tempest_plugin.common import ssh
from neutron_tempest_plugin import config
from neutron_tempest_plugin.scenario import base

CONF = config.CONF
LOG = log.getLogger(__name__)


class InternalDNSBase(base.BaseTempestTestCase):
    """Base class of useful resources and functionalities for test class."""

    port_error_msg = ('Openstack command returned incorrect '
                      'hostname value in port.')
    ssh_error_msg = ('Remote shell command returned incorrect hostname value '
                     "(command: 'hostname' OR 'cat /etc/hostname').")

    @staticmethod
    def _rand_name(name):
        """'data_utils.rand_name' wrapper, show name related to test suite."""
        return data_utils.rand_name(f'internal-dns-test-{name}')

    @classmethod
    def resource_setup(cls):
        super(InternalDNSBase, cls).resource_setup()
        cls.router = cls.create_router_by_client()
        cls.keypair = cls.create_keypair(
            name=cls._rand_name('shared-keypair'))
        cls.secgroup = cls.create_security_group(
            name=cls._rand_name('shared-secgroup'))
        cls.security_groups.append(cls.secgroup)
        cls.create_loginable_secgroup_rule(
            secgroup_id=cls.secgroup['id'])
        cls.vm_kwargs = {
            'flavor_ref': CONF.compute.flavor_ref,
            'image_ref': CONF.compute.image_ref,
            'key_name': cls.keypair['name'],
            'security_groups': [{'name': cls.secgroup['name']}]
        }

    def _create_ssh_client(self, ip_addr):
        return ssh.Client(ip_addr,
                          CONF.validation.image_ssh_user,
                          pkey=self.keypair['private_key'])

    def _validate_port_dns_details(self, checked_hostname, checked_port):
        """Validates reused objects for correct dns values in tests."""
        dns_details = checked_port['dns_assignment'][0]
        self.assertEqual(checked_hostname, checked_port['dns_name'],
                         self.port_error_msg)
        self.assertEqual(checked_hostname, dns_details['hostname'],
                         self.port_error_msg)
        self.assertIn(checked_hostname, dns_details['fqdn'],
                      self.port_error_msg)

    def _validate_ssh_dns_details(self, checked_hostname, ssh_client):
        """Validates correct dns values returned from ssh command in tests."""
        ssh_output = ssh_client.get_hostname()
        self.assertIn(checked_hostname, ssh_output, self.ssh_error_msg)


class InternalDNSTest(InternalDNSBase):
    """Tests internal DNS capabilities."""
    credentials = ['primary', 'admin']

    @utils.requires_ext(extension="dns-integration", service="network")
    @decorators.idempotent_id('988347de-07af-471a-abfa-65aea9f452a6')
    def test_dns_domain_and_name(self):
        """Test the ability to ping a VM's hostname from another VM.

        1) Create two VMs on the same network, giving each a name
        2) SSH in to the first VM:
          2.1) ping the other VM's internal IP
          2.2) ping the other VM's hostname
        """
        network = self.create_network(dns_domain='starwars.')
        self.setup_network_and_server(network=network, server_name='luke')
        self.create_pingable_secgroup_rule(
            secgroup_id=self.security_groups[-1]['id'])
        self.check_connectivity(self.fip['floating_ip_address'],
                                CONF.validation.image_ssh_user,
                                self.keypair['private_key'])

        leia = self.create_server(
            flavor_ref=CONF.compute.flavor_ref,
            image_ref=CONF.compute.image_ref,
            key_name=self.keypair['name'],
            networks=[{'uuid': self.network['id']}],
            security_groups=[
                {'name': self.security_groups[-1]['name']}],
            name='leia')

        ssh_client = ssh.Client(
            self.fip['floating_ip_address'],
            CONF.validation.image_ssh_user,
            pkey=self.keypair['private_key'])

        self.assertIn('luke', ssh_client.get_hostname())

        leia_port = self.client.list_ports(
            network_id=self.network['id'],
            device_id=leia['server']['id'])['ports'][0]

        # Ping with a higher timeout because spawning 2 VMs in some
        # environment can put significant load on the deployment, resulting
        # in very long boot times.
        self.check_remote_connectivity(
            ssh_client, leia_port['fixed_ips'][0]['ip_address'],
            timeout=CONF.validation.ping_timeout * 10,
            servers=[self.server, leia])

        resolv_conf = ssh_client.exec_command('cat /etc/resolv.conf')
        dns_domain = CONF.neutron_plugin_options.dns_domain
        self.assertIn(dns_domain, resolv_conf)
        self.assertNotIn('starwars', resolv_conf)

        self.check_remote_connectivity(ssh_client, 'leia',
                                       servers=[self.server, leia])
        self.check_remote_connectivity(ssh_client, 'leia.' + dns_domain,
                                       servers=[self.server, leia])

    @utils.requires_ext(extension="dns-integration", service="network")
    @decorators.idempotent_id('db5e612f-f17f-4974-b5f1-9fe89f4a6fc9')
    def test_create_and_update_port_with_dns_name(self):
        """Test creation of port with correct internal dns-name (hostname)."""

        # 1) Create resources: network, subnet, etc.
        # 2) Create a port with wrong dns-name (not as VM name).
        # 3) Verify that wrong port initial dns-name.
        #    was queried from openstack API.
        # 4) Update the port with correct dns-name (as VM name).
        # 5) Boot a VM with corrected predefined port.
        # 6) Verify that correct port dns-name
        #    was queried from openstack API.
        # 7) Validate hostname configured on VM is same as VM's name.

        # NOTE: VM's hostname has to be the same as VM's name
        #       when a VM is created, it is a known limitation.
        #       Therefore VM's dns-name/hostname is checked to be as VM's name.

        vm_correct_name = self._rand_name('vm')
        vm_wrong_name = self._rand_name('bazinga')
        # create resources
        network = self.create_network(name=self._rand_name('network'))
        subnet = self.create_subnet(network, name=self._rand_name('subnet'))
        self.create_router_interface(self.router['id'], subnet['id'])
        # create port with wrong dns-name (not as VM name)
        dns_port = self.create_port(network,
                                    dns_name=vm_wrong_name,
                                    security_groups=[self.secgroup['id']],
                                    name=self._rand_name('port'))
        # validate dns port with wrong initial hostname from API
        self._validate_port_dns_details(vm_wrong_name, dns_port)
        # update port with correct dns-name (as VM name)
        dns_port = self.update_port(dns_port, dns_name=vm_correct_name)
        # create VM with correct predefined dns-name on port
        vm_1 = self.create_server(name=vm_correct_name,
                                  networks=[{'port': dns_port['id']}],
                                  **self.vm_kwargs)
        # validate dns port with correct changed hostname using API
        self._validate_port_dns_details(vm_correct_name, dns_port)
        # validate hostname configured on VM is same as VM's name.
        vm_1['fip'] = self.create_floatingip(port=dns_port)
        vm_1['ssh_client'] = self._create_ssh_client(
            vm_1['fip']['floating_ip_address'])
        self._validate_ssh_dns_details(vm_correct_name, vm_1['ssh_client'])



# Copyright 2016 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_lib.api.definitions import provider_net
from oslo_log import log
from oslo_serialization import jsonutils
from tempest.common import utils
from tempest.common import waiters
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
import testtools

from neutron_tempest_plugin.common import ssh
from neutron_tempest_plugin import config
from neutron_tempest_plugin.scenario import base
from neutron_tempest_plugin.scenario import constants

CONF = config.CONF
LOG = log.getLogger(__name__)


class NetworkMtuBaseTest(base.BaseTempestTestCase):

    @classmethod
    def resource_setup(cls):
        super(NetworkMtuBaseTest, cls).resource_setup()
        # setup basic topology for servers we can log into it
        cls.router = cls.create_router_by_client()
        cls.keypair = cls.create_keypair()
        cls.secgroup = cls.os_primary.network_client.create_security_group(
            name='secgroup_mtu')
        cls.security_groups.append(cls.secgroup['security_group'])
        cls.create_loginable_secgroup_rule(
            secgroup_id=cls.secgroup['security_group']['id'])
        cls.create_pingable_secgroup_rule(
            secgroup_id=cls.secgroup['security_group']['id'])
        if CONF.neutron_plugin_options.default_image_is_advanced:
            cls.use_advanced_image = False
            cls.username = CONF.validation.image_ssh_user
        else:
            cls.use_advanced_image = True
            cls.username = CONF.neutron_plugin_options.advanced_image_ssh_user

    def create_pingable_vm(self, net, keypair, secgroup):
        if self.use_advanced_image:
            flavor_ref = CONF.neutron_plugin_options.advanced_image_flavor_ref
            image_ref = CONF.neutron_plugin_options.advanced_image_ref
        else:
            flavor_ref = CONF.compute.flavor_ref
            image_ref = CONF.compute.image_ref

        server = self.create_server(
            flavor_ref=flavor_ref,
            image_ref=image_ref,
            key_name=keypair['name'],
            networks=[{'uuid': net['id']}],
            security_groups=[{'name': secgroup[
                'security_group']['name']}])
        waiters.wait_for_server_status(
            self.os_primary.servers_client, server['server']['id'],
            constants.SERVER_STATUS_ACTIVE)
        port = self.client.list_ports(
            network_id=net['id'], device_id=server['server']['id'])['ports'][0]
        fip = self.create_floatingip(port=port)
        return server, fip

    def _get_network_params(self):
        return jsonutils.loads(CONF.neutron_plugin_options.test_mtu_networks)


class NetworkMtuTest(NetworkMtuBaseTest):
    credentials = ['primary', 'admin']
    servers = []
    networks = []

    @classmethod
    def skip_checks(cls):
        super(NetworkMtuTest, cls).skip_checks()
        if ("vxlan" not in
                config.CONF.neutron_plugin_options.available_type_drivers or
            "gre" not in
                config.CONF.neutron_plugin_options.available_type_drivers):
            raise cls.skipException("GRE or VXLAN type_driver is not enabled")

    @classmethod
    @utils.requires_ext(extension=provider_net.ALIAS, service="network")
    def resource_setup(cls):
        super(NetworkMtuTest, cls).resource_setup()

    def _create_setup(self):
        self.admin_client = self.os_admin.network_client
        net_kwargs = {'tenant_id': self.client.tenant_id}
        for net_type in ['vxlan', 'gre']:
            net_kwargs['name'] = '-'.join([net_type, 'net'])
            net_kwargs['provider:network_type'] = net_type
            network = self.admin_client.create_network(**net_kwargs)[
                'network']
            self.networks.append(network)
            self.addCleanup(self.admin_client.delete_network, network['id'])
            subnet = self.create_subnet(network)
            self.create_router_interface(self.router['id'], subnet['id'])
            self.addCleanup(self.client.remove_router_interface_with_subnet_id,
                            self.router['id'], subnet['id'])
        # check that MTUs are different for 2 networks
        self.assertNotEqual(self.networks[0]['mtu'], self.networks[1]['mtu'])
        self.networks.sort(key=lambda net: net['mtu'])
        server1, fip1 = self.create_pingable_vm(self.networks[0],
                                                self.keypair, self.secgroup)
        server_ssh_client1 = ssh.Client(
            self.floating_ips[0]['floating_ip_address'],
            self.username, pkey=self.keypair['private_key'])
        server2, fip2 = self.create_pingable_vm(self.networks[1],
                                                self.keypair, self.secgroup)
        server_ssh_client2 = ssh.Client(
            self.floating_ips[0]['floating_ip_address'],
            self.username, pkey=self.keypair['private_key'])
        for fip in (fip1, fip2):
            self.check_connectivity(
                fip['floating_ip_address'],
                self.username, self.keypair['private_key'],
                servers=[server1, server2])
        return server_ssh_client1, fip1, server_ssh_client2, fip2

    @testtools.skipUnless(
        (CONF.neutron_plugin_options.advanced_image_ref or
         CONF.neutron_plugin_options.default_image_is_advanced),
        "Advanced image is required to run this test.")
    @decorators.idempotent_id('3d73ec1a-2ec6-45a9-b0f8-04a273d9d344')
    def test_connectivity_min_max_mtu(self):
        server_ssh_client, _, _, fip2 = self._create_setup()
        # ping with min mtu of 2 networks succeeds even when
        # fragmentation is disabled
        self.check_remote_connectivity(
            server_ssh_client, fip2['fixed_ip_address'],
            mtu=self.networks[0]['mtu'], fragmentation=False)

        # ping with the size above min mtu of 2 networks
        # fails when fragmentation is disabled
        self.check_remote_connectivity(
            server_ssh_client, fip2['fixed_ip_address'], should_succeed=False,
            mtu=self.networks[0]['mtu'] + 1, fragmentation=False)

        # ping with max mtu of 2 networks succeeds when
        # fragmentation is enabled
        self.check_remote_connectivity(
            server_ssh_client, fip2['fixed_ip_address'],
            mtu=self.networks[1]['mtu'])

        # ping with max mtu of 2 networks fails when fragmentation is disabled
        self.check_remote_connectivity(
            server_ssh_client, fip2['fixed_ip_address'], should_succeed=False,
            mtu=self.networks[1]['mtu'], fragmentation=False)


class NetworkWritableMtuTest(NetworkMtuBaseTest):
    credentials = ['primary', 'admin']
    servers = []
    networks = []

    @classmethod
    def skip_checks(cls):
        super(NetworkWritableMtuTest, cls).skip_checks()
        supported_type_drivers = ['vxlan', 'geneve']
        if not any(type_driver in supported_type_drivers for type_driver in
                   config.CONF.neutron_plugin_options.available_type_drivers):
            raise cls.skipException(
                "Neither VXLAN nor GENEVE type_driver is enabled")

    @classmethod
    @utils.requires_ext(extension="net-mtu-writable", service="network")
    def resource_setup(cls):
        super(NetworkWritableMtuTest, cls).resource_setup()

    def _create_setup(self):
        self.admin_client = self.os_admin.network_client
        for test_net in self._get_network_params():
            test_net['tenant_id'] = self.client.tenant_id
            test_net['name'] = data_utils.rand_name('net')
            cidr = None if 'cidr' not in test_net else test_net.pop('cidr')
            network = self.admin_client.create_network(**test_net)[
                'network']
            self.networks.append(network)
            self.addCleanup(self.admin_client.delete_network, network['id'])
            subnet = self.create_subnet(network, cidr=cidr)
            self.create_router_interface(self.router['id'], subnet['id'])
            self.addCleanup(self.client.remove_router_interface_with_subnet_id,
                            self.router['id'], subnet['id'])

        # update network mtu
        net_mtu = self.admin_client.show_network(
            self.networks[0]['id'])['network']['mtu']
        self.admin_client.update_network(self.networks[0]['id'],
                                         mtu=(net_mtu - 1))
        self.networks[0]['mtu'] = (
            self.admin_client.show_network(
                self.networks[0]['id'])['network']['mtu'])

        # check that MTUs are different for 2 networks
        self.assertNotEqual(self.networks[0]['mtu'], self.networks[1]['mtu'])
        self.networks.sort(key=lambda net: net['mtu'])
        server1, fip1 = self.create_pingable_vm(self.networks[0],
                                                self.keypair, self.secgroup)
        server_ssh_client1 = ssh.Client(
            self.floating_ips[0]['floating_ip_address'],
            self.username, pkey=self.keypair['private_key'])
        server2, fip2 = self.create_pingable_vm(self.networks[1],
                                                self.keypair, self.secgroup)
        server_ssh_client2 = ssh.Client(
            self.floating_ips[0]['floating_ip_address'],
            self.username, pkey=self.keypair['private_key'])
        for fip in (fip1, fip2):
            self.check_connectivity(
                fip['floating_ip_address'],
                self.username, self.keypair['private_key'])
        return server_ssh_client1, fip1, server_ssh_client2, fip2

    @testtools.skipUnless(
        (CONF.neutron_plugin_options.advanced_image_ref or
         CONF.neutron_plugin_options.default_image_is_advanced),
        "Advanced image is required to run this test.")
    @decorators.idempotent_id('bc470200-d8f4-4f07-b294-1b4cbaaa35b9')
    def test_connectivity_min_max_mtu(self):
        server_ssh_client, _, _, fip2 = self._create_setup()
        log_msg = ("Ping with {mtu_size} MTU of 2 networks. Fragmentation is "
                   "{fragmentation_state}. Expected result: ping "
                   "{ping_status}")

        # ping with min mtu of 2 networks succeeds even when
        # fragmentation is disabled
        LOG.debug(log_msg.format(mtu_size='minimal',
                  fragmentation_state='disabled', ping_status='succeeded'))
        self.check_remote_connectivity(
            server_ssh_client, fip2['fixed_ip_address'],
            mtu=self.networks[0]['mtu'], fragmentation=False)

        # ping with the size above min mtu of 2 networks
        # fails when fragmentation is disabled
        LOG.debug(log_msg.format(mtu_size='size above minimal',
                  fragmentation_state='disabled', ping_status='failed'))
        self.check_remote_connectivity(
            server_ssh_client, fip2['fixed_ip_address'], should_succeed=False,
            mtu=self.networks[0]['mtu'] + 2, fragmentation=False)

        # ping with max mtu of 2 networks succeeds when
        # fragmentation is enabled
        LOG.debug(log_msg.format(mtu_size='maximal',
                  fragmentation_state='enabled', ping_status='succeeded'))
        self.check_remote_connectivity(
            server_ssh_client, fip2['fixed_ip_address'],
            mtu=self.networks[1]['mtu'])

        # ping with max mtu of 2 networks fails when fragmentation is disabled
        LOG.debug(log_msg.format(mtu_size='maximal',
                  fragmentation_state='disabled', ping_status='failed'))
        self.check_remote_connectivity(
            server_ssh_client, fip2['fixed_ip_address'], should_succeed=False,
            mtu=self.networks[1]['mtu'], fragmentation=False)



# Copyright 2013 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_lib import constants
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

from neutron_tempest_plugin.api import base


class ExtraDHCPOptionsTestJSON(base.BaseNetworkTest):
    """Test Extra DHCP Options

    Tests the following operations with the Extra DHCP Options Neutron API
    extension:

        port create
        port list
        port show
        port update

    v2.0 of the Neutron API is assumed. It is also assumed that the Extra
    DHCP Options extension is enabled in the [network-feature-enabled]
    section of etc/tempest.conf
    """

    required_extensions = ['extra_dhcp_opt']

    @classmethod
    def resource_setup(cls):
        super(ExtraDHCPOptionsTestJSON, cls).resource_setup()
        cls.network = cls.create_network()
        cls.subnet = cls.create_subnet(cls.network)
        cls.port = cls.create_port(cls.network)
        cls.ip_tftp = ('123.123.123.123' if cls._ip_version == 4
                       else '2015::dead')
        cls.ip_server = ('123.123.123.45' if cls._ip_version == 4
                         else '2015::badd')
        cls.extra_dhcp_opts = [
            {'opt_value': 'pxelinux.0',
             'opt_name': 'bootfile-name'},  # default ip_version is 4
            {'opt_value': cls.ip_tftp,
             'opt_name': 'tftp-server',
             'ip_version': cls._ip_version},
            {'opt_value': cls.ip_server,
             'opt_name': 'server-ip-address',
             'ip_version': cls._ip_version}
        ]

    @decorators.idempotent_id('d2c17063-3767-4a24-be4f-a23dbfa133c9')
    def test_create_list_port_with_extra_dhcp_options(self):
        # Create a port with Extra DHCP Options
        body = self.create_port(
            self.network,
            extra_dhcp_opts=self.extra_dhcp_opts)
        port_id = body['id']

        # Confirm port created has Extra DHCP Options
        body = self.client.list_ports()
        ports = body['ports']
        port = [p for p in ports if p['id'] == port_id]
        self.assertTrue(port)
        self._confirm_extra_dhcp_options(port[0], self.extra_dhcp_opts)

    @decorators.idempotent_id('9a6aebf4-86ee-4f47-b07a-7f7232c55607')
    def test_update_show_port_with_extra_dhcp_options(self):
        # Update port with extra dhcp options
        name = data_utils.rand_name('new-port-name')
        body = self.client.update_port(
            self.port['id'],
            name=name,
            extra_dhcp_opts=self.extra_dhcp_opts)
        # Confirm extra dhcp options were added to the port
        body = self.client.show_port(self.port['id'])
        self._confirm_extra_dhcp_options(body['port'], self.extra_dhcp_opts)

    def _confirm_extra_dhcp_options(self, port, extra_dhcp_opts):
        retrieved = port['extra_dhcp_opts']
        self.assertEqual(len(retrieved), len(extra_dhcp_opts))
        for retrieved_option in retrieved:
            for option in extra_dhcp_opts:
                # default ip_version is 4
                ip_version = option.get('ip_version', constants.IP_VERSION_4)
                if (retrieved_option['opt_value'] == option['opt_value'] and
                    retrieved_option['opt_name'] == option['opt_name'] and
                    retrieved_option['ip_version'] == ip_version):
                    break
            else:
                self.fail('Extra DHCP option not found in port %s' %
                          str(retrieved_option))


class ExtraDHCPOptionsIpV6TestJSON(ExtraDHCPOptionsTestJSON):
    _ip_version = 6
